@@ -416,6 +416,7 @@ static void xc_fillinfo_dmz(int cachetype, xc_cache_t *cache, zval *return_value
 	add_assoc_long_ex(return_value, ZEND_STRS("hits"),      cache->hits);
 	add_assoc_long_ex(return_value, ZEND_STRS("clogs"),     cache->clogs);
 	add_assoc_long_ex(return_value, ZEND_STRS("ooms"),      cache->ooms);
+	add_assoc_long_ex(return_value, ZEND_STRS("errors"),    cache->errors);
 
 	add_assoc_long_ex(return_value, ZEND_STRS("cached"),    cache->entries_count);
 	add_assoc_long_ex(return_value, ZEND_STRS("deleted"),   cache->deletes_count);
@@ -546,6 +547,7 @@ static zend_op_array *xc_entry_install(xc_entry_t *xce, zend_file_handle *h TSRM
 {
 	zend_uint i;
 	xc_entry_data_php_t *p = xce->data.php;
+	zend_op_array *old_active_op_array = CG(active_op_array);
 #ifndef ZEND_ENGINE_2
 	/* new ptr which is stored inside CG(class_table) */
 	xc_cest_t **new_cest_ptrs = (xc_cest_t **)do_alloca(sizeof(xc_cest_t*) * p->classinfo_cnt);
@@ -611,6 +613,7 @@ static zend_op_array *xc_entry_install(xc_entry_t *xce, zend_file_handle *h TSRM
 #ifndef ZEND_ENGINE_2
 	free_alloca(new_cest_ptrs);
 #endif
+	CG(active_op_array) = old_active_op_array;
 	return p->op_array;
 }
 /* }}} */
@@ -715,6 +718,7 @@ static int xc_entry_init_key_php(xc_entry_t *xce, char *filename, char *opened_p
 	int cacheid;
 	xc_entry_data_php_t *php;
 	char *ptr;
+	time_t delta;
 
 	if (!filename || !SG(request_info).path_translated) {
 		return 0;
@@ -765,7 +769,8 @@ not_relative_path:
 		/* fall */
 
 stat_done:
-		if (XG(request_time) - pbuf->st_mtime < 2 && !xc_test) {
+		delta = XG(request_time) - pbuf->st_mtime;
+		if (abs(delta) < 2 && !xc_test) {
 			return 0;
 		}
 
@@ -964,8 +969,13 @@ static zend_op_array *xc_compile_file(zend_file_handle *h, int type TSRMLS_DC) /
 
 	if (!XG(initial_compile_file_called)) {
 		xc_sandbox_free(&sandbox, XC_InstallNoBinding TSRMLS_CC);
-        return op_array;
-    }
+		ENTER_LOCK(cache) {
+			cache->compiling = 0;
+			/* it's not cachable, but don't scare the users with high misses */
+			cache->misses --;
+		} LEAVE_LOCK(cache);
+		return op_array;
+	}
 
 	filename = h->opened_path ? h->opened_path : h->filename;
 	/* none-inode enabled entry hash/compare on name
@@ -1144,14 +1154,17 @@ err_bailout:
 		xc_sandbox_free(&sandbox, XC_NoInstall TSRMLS_CC);
 	}
 	else {
+		zend_op_array *old_active_op_array = CG(active_op_array);
 		CG(active_op_array) = op_array;
 		xc_sandbox_free(&sandbox, XC_Install TSRMLS_CC);
+		CG(active_op_array) = old_active_op_array;
 	}
 
 	ENTER_LOCK(cache) {
 		cache->compiling = 0;
 	} LEAVE_LOCK(cache);
 	if (catched) {
+		cache->errors ++;
 		zend_bailout();
 	}
 	if (xc_test && stored_xce) {
@@ -1449,11 +1462,12 @@ static int xc_init(int module_number TSRMLS_DC) /* {{{ */
 	return SUCCESS;
 
 err:
-	xc_destroy();
 	if (xc_php_caches || xc_var_caches) {
+		xc_destroy();
 		/* shm destroied in xc_destroy() */
 	}
 	else if (shm) {
+		xc_destroy();
 		xc_shm_destroy(shm);
 	}
 	return 0;
@@ -2746,10 +2760,27 @@ static ZEND_MODULE_POST_ZEND_DEACTIVATE_D(xcache)
 	return SUCCESS;
 }
 /* }}} */
+/* {{{ module dependencies */
+#if ZEND_MODULE_API_NO >= 20050922
+static const zend_module_dep xcache_module_deps[] = {
+	ZEND_MOD_REQUIRED("standard")
+	ZEND_MOD_CONFLICTS("apc")
+	ZEND_MOD_CONFLICTS("eAccelerator")
+	ZEND_MOD_CONFLICTS("Turck MMCache")
+	{NULL, NULL, NULL}
+};
+#endif
+/* }}} */ 
 /* {{{ module definition structure */
 
 zend_module_entry xcache_module_entry = {
+#if ZEND_MODULE_API_NO >= 20050922
+	STANDARD_MODULE_HEADER_EX,
+	NULL,
+	xcache_module_deps,
+#else
 	STANDARD_MODULE_HEADER,
+#endif
 	XCACHE_NAME,
 	xcache_functions,
 	PHP_MINIT(xcache),
